package Gear::Remotes::Watch;

use strict;
use warnings;
use Gear::Rules;
use Gear::Remotes;

use IPC::Open2;

our $VERSION=0.001;
our $verbose=0;
my $debug=0;

sub ls_git_tags_filtered {
    my $topdir=Gear::Git::Helper::get_git_toplevel();
    my $cmd='git tag';
    my $filter_tag_file=$topdir.'.gear/upstream/filter-tag';
    my $has_filter_tag=-x $filter_tag_file;
    my $transform_tag_file=$topdir.'.gear/upstream/transform-tag';
    my $has_transform_tag=-x $transform_tag_file;
    #warn "DEBUG: has_transform_tag=",$has_transform_tag," has_filter_tag=",$has_filter_tag,"\n" if $debug;
    foreach my $bad_file ($topdir.'.gear/upstream/filter_tag', $topdir.'.gear/upstream/transform_tag') {
	if (-e $bad_file) {
	    warn "WARNING: bad name for remotes: $bad_file. Should be *-tag, not *_tag.\n"
	}
    }
    $cmd .= '| '.$filter_tag_file if $has_filter_tag;
    open (my $fn, '-|',$cmd) or die "can't run $cmd: $!";
    my @git_tag;
    while (<$fn>) {
	chomp;
	next if not $has_filter_tag and not /^[Vv]?[\d\.]*\d(?:-rel|-release)?$/;
	push @git_tag, $_;
    }
    close($fn);
    my @git_tag_pairs;
    if (not $has_transform_tag) {
	foreach my $tag (@git_tag) {
	    my $ver=$tag;
	    $ver=~s/^[vV](\d)/$1/;
	    $ver=~s/(?:-rel|-release)$//;
	    push @git_tag_pairs, [$ver,$tag];
	}
    } else {
	my($chld_out, $chld_in, $pid);
	$pid = open2($chld_out, $chld_in, $transform_tag_file);
	print $chld_in map {"$_\n"} @git_tag;
	close($chld_in);
	foreach my $tag (@git_tag) {
	    my $ver=<$chld_out>;
	    if (not defined($ver) or $ver=~/^\s*$/) {
		die "Gear::Remotes::Watch: bad filter $transform_tag_file: tried to transform $tag, got nothing";
	    }
	    chomp $ver;
	    push @git_tag_pairs, [$ver,$tag];
	}
	close($chld_out);
	waitpid( $pid, 0 );
	my $child_exit_status = $? >> 8;
    }
    #warn "DEBUG: git_tag_pairs:\n", map{"$_->[0] -> $_->[1]\n"} @git_tag_pairs if $debug;
    return @git_tag_pairs;
}

sub find_tag_for_version {
    my ($version)=@_;
    foreach my $pair (&Gear::Remotes::Watch::ls_git_tags_filtered()) {
	my ($ver,$tag)=@$pair;
	return $tag if $ver eq $version;
    }
    return;
}

# TODO: move Versort somewhere
use RPM::uscan;

my %watch_remotes_known_opts=map {$_=>1} qw/
    -action
    -sysexit
    -verbose
    -error_if_ahead
    -watch_format_version
/;

# we expect utils to support debian watch version2 format (--upstream-version option)
my $watch_format_version=2;

sub watch_remotes {
    my %opt=(
	-watch_format_version => $watch_format_version,
	@_,
    );
    foreach my $option (keys(%opt)) {
	die "unsupported option $option" unless $watch_remotes_known_opts{$option};
    }
    my $verbose=$opt{-verbose};
    my ($version_cmp, $oldversion, $newversion, $target)=&tagwatch_versions(undef);
    if ($version_cmp>1) {
	if ($opt{-error_if_ahead}) {
	    print STDERR "Oops! local repository ($oldversion) is ahead of remote repository ($newversion from $target).\n";
	} elsif ($verbose) {
	    print "Local repository ($oldversion) is ahead of remote repository ($newversion from $target).\n";
	}
	exit 21 if $opt{-sysexit};
    } elsif ($version_cmp==0) {
	print "Repository is up to date.\n" if $verbose;
	exit 22 if $opt{-sysexit};
    } else {
	if ($verbose) {
	    print "New version $newversion is available from tag $target.\n";
	} else {
	    print "$newversion\t$target\n";
	}
	exit 23 if $opt{-sysexit};
	my $action=$opt{-action};
	if ($action) {
	    print "Running $action [ $oldversion -> $newversion ] from tag $target ...\n" if $verbose;
	    RPM::uscan::execute_user_action ($action, $opt{-watch_format_version}, $newversion, $target);
	}
    }
}

sub tagwatch_versions {
    my $gear_remotes=Gear::Remotes->new();
    $gear_remotes->fetch();
    my @tag_pairs=Gear::Remotes::Watch::ls_git_tags_filtered();
    die "Gear::Remotes: no matching tags found\n" unless @tag_pairs;
    my @sorted = Devscripts::Versort::upstream_versort(@tag_pairs);
    my $ver=$sorted[0]->[0];
    my ($upackage, $uversion) = &__gear__describe();
    my $version_cmp=Devscripts::Versort::myvercmp($uversion, $ver);
    return ($version_cmp,$uversion, $ver,$sorted[0]->[1]);
}

sub __gear__describe {
    my $gdescribe=`gear --describe`;
    unless ($?) {
	chomp $gdescribe;
	my @gear_describe=split(/\s+/,$gdescribe);
	return @gear_describe;
    } else {
	die "gear --describe failed.";
    }
}

1;
